program bench
  !
  ! bench marking for sparse matrix multiply A*A^T
  !
  use basics
  use csr_matrix_mod
  use cpu_mod

  ! input unit 
  integer (kind=myint) :: input_unit
  integer (kind = myint) :: ierr

  ! input file name
  character (len=120):: infile
  integer :: i1,i2,i,nz1,j,ilen
  logical :: ex

  ! the matrix and right-hand-side.
  integer (kind=myint) :: nz
  integer (kind=myint) :: n,m
  integer (kind=myint), pointer :: irn(:)
  integer (kind=myint), pointer :: jcn(:)
  real (kind = myreal), pointer :: val(:)

  type (csr_matrix) :: b1,b2
  real (kind = myreal), dimension (:), pointer :: a,c
  integer (kind = myint), dimension (:), pointer :: ia,ja,ic,jc

  ! index base (for fortran, it starts from 1) used in C
  integer (kind = myint) :: ind_base

  real (kind = myreal4) :: time1,time2
  integer ::  spmatmul_size
  real (kind = myreal) :: avg
  ! first character in the matrix file
  character (len = 1) firstc

  ! the minmum CPU time (in seconds) needed to
  ! run benchmark so as to get a good average timing

  real (kind = myreal),parameter :: CPU_MIN = 2.0

  ! flops: number of floating point operations per run
  integer (kind = myint) :: flops

! for Cray  call pxfgetarg(1,infile,i1,i2)
  call getarg(1,infile)

  if (Ichar(infile(1:1)) == 32) then
     stop "Usage should be: bench <matrix_file>"
  endif

  ! F90 IO -----------------------------

  ! get rid of left space, find the true length, check that the
  ! file exist
  ! and open the graph file
  infile=adjustl(infile)
  ilen=len_trim(infile)
  input_unit = 10
  inquire(file=infile(1:ilen),exist=ex)
  if (ex) then
     open(unit = input_unit, file=infile(1:ilen), err = 99)
  else
     write(*,*) "matrix file: ",infile(1:ilen),"does not exist"
     stop
  end if
  goto 199
99 write(*,*) "try to open file ",infile(1:ilen)
  stop "problem opening matrix file"
     
199 continue


  nrun = 0
  IO_BENCH: do 
     ! get rid of comments in the form %...
     firstc = "%"
     do while (firstc(1:1) == "%")
        read(input_unit,"(a)") firstc
     end do
     backspace(input_unit)

     ! read in the row, column sizes and number of nonzeros
     read(input_unit,*) n,m,nz
     allocate(irn(nz),jcn(nz),val(nz))
     
     if (nrun == 0) time1 = cpu()
     ! read in the row and column indices (sparse pattern)
     nrun = nrun + 1
     do i = 1, nz
        read(input_unit,*) irn(i),jcn(i)
     end do
     time2 = cpu()
     if (time2-time1 > CPU_MIN) exit
     rewind(input_unit)
     deallocate(irn,jcn,val)
  end do IO_BENCH
  close(input_unit)

  write(*,*) " =========== F90 benchmark for sparse multiplication ============"
  write(*,*) " case name = ",infile(1:ilen)
  write(*,*) " problem size = ",n
  write(*,*) " nz = ",nz
  write(*,*) " time_IO = ",(time2 - time1)/nrun," averaged over ",nrun," runs"
  val = 1.0
  call coo_to_csr_format(n,nz,irn,jcn,val,b1,structure_only = .false.)
  ia => b1%ia
  ja => b1%ja
  a => b1%a

  ! F90 multiply ------------------------------

  time1 = cpu()
  nrun = 0
  COMP_BENCH: do
     nrun = nrun + 1
     call matrix_multiply(b1,b1,b2,b1%size_column)
     time2 = cpu()
     if (time2-time1 > CPU_MIN) exit
     call matrix_deallocate(b2)
  end do COMP_BENCH

  nz1 = b2%ia(n+1)-1
  write(*,*)  " nz1 = ",nz1
  write(*,*) " time_COMP = ",(time2 - time1)/nrun," averaged over ",nrun," runs"
  flops = matrix_multiply_flops(b1%size,b1%size_column,b1%ia,b1%ja,b1%ia,b1%ja)
  write(*,*) " MFlops/s = ",nrun*flops/1000000.0/(time2 - time1)
  write(*,*) " flops/run = ",flops
  
  ! average entry value
  avg = 0
  do i = 1, nz1
     avg = avg + b2%a(i)
  end do
  write(*,"(a,g26.16)") "  average entry value = ",avg/nz1

end program bench

!!$! if gerarg is not available
!!$subroutine getarg(i,file)
!!$  use basics
!!$  character (len = 120) :: file
!!$  write(*,*) "what is the file name "
!!$  read(*,*) file
!!$  write(*,*) "file to be open = ",file
!!$end subroutine getarg







